<?php
/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author PrestaShop SA <contact@prestashop.com>
 *  @copyright  2007-2020 PrestaShop SA
 *  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class ConnectixCallbackModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();
        $this->ajax = true;
    }

    public function displayAjax()
    {
        header('Content-Type: application/json');
        $paramSecret = Tools::getValue('secret');
        $secret = Configuration::get('CONNECTIX_SECRET');
        $protocol = (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0');

        $type = Tools::getValue('type');
        if (!$type) {
            header("$protocol 400 Bad Request");
            echo "some parameters are missing";
            exit;
        }

        $isSpecialRoute = in_array($type, ['confirm', 'cancel']);
        if (!(int)Configuration::get('CONNECTIX_ENABLED') || (!$isSpecialRoute && (empty($paramSecret) || $paramSecret !== $secret)) || ($isSpecialRoute && empty(Tools::getValue('token')))) {
            header("$protocol 403 Forbidden");
            echo "Forbidden";
            exit;
        }

        $data = $this->getPayload();
        if (!$isSpecialRoute && is_null($data)) {
            header("$protocol 400 Bad Request");
            echo "The payload is missing or malformed";
            exit;
        }

        $action = null;
        if (isset($data['action'])) {
            $action = $data['action'];
            unset($data['action']);
        }

        switch ($type) {
            case 'callback':
                $success = $this->handleCallback($data);
                break;
            case 'inbound':
                $success = $this->handleInbound($data);
                break;
            case 'template':
                $success = $this->handleTemplate($data);
                break;
            case 'tracking':
                if ($action === 'readyForDelivery') {
                    $success = $this->handleReadyForDelivery($data);
                } elseif ($action === 'delivered') {
                    $success = $this->handleDelivered($data);
                } elseif ($action === 'notPickedUp') {
                    $success = $this->handleNotPickedUp($data);
                }
                break;
            case 'confirm':
            case 'cancel':
                $this->handleOrderConfirmation($type, Tools::getValue('token'));
                $success = true;
                break;
            default:
                $success = false;
                break;
        }

        if ($success) {
            echo json_encode("OK");
        } else {
            header("$protocol 400 Bad Request");
            echo json_encode("Something is wrong..");
        }
        exit;
    }

    /**
     * @return mixed|null
     */
    private function getPayload()
    {
        $payload = Tools::file_get_contents('php://input');
        if (empty($payload)) {
            return null;
        }

        return json_decode($payload, true);
    }

    /**
     * @param $expected
     * @param $data
     * @return bool
     */
    private function validateData($expected, $data)
    {
        $valid = true;
        foreach ($expected as $field) {
            if (!array_key_exists($field, $data)) {
                $valid = false;
                break;
            }
        }
        return $valid;
    }

    /**
     * @param array $data
     * @return false
     */
    private function handleCallback(array $data)
    {
        if (!$this->validateData(['id', 'status', 'channel', 'createdAt', 'fallbackId'],  $data) && !$this->validateData(['id', 'status', 'channel', 'createdAt'],  $data)) {
            return false;
        }

        $message = DB::getInstance()->getRow('SELECT id,fallback_id FROM '._DB_PREFIX_.'connectix_messages WHERE provider_id = "'.pSQL($data['id']).'"');
        if (!$message) {
            return true;
        }

        Db::getInstance()->update(
            'connectix_messages',
            [
                'channel' => pSQL($data['channel']),
                'status' => pSQL($data['status']),
            ],
            'provider_id = "'.pSQL($data['id']).'"'
        );

        if (in_array($data['status'], ['delivered', 'seen']) && $message['fallback_id']) {
            Db::getInstance()->delete('connectix_messages', 'status = "pending" AND fallback_id = "'.pSQL($message['fallback_id']).'"');
        }

        return true;
    }

    /**
     * @param array $data
     * @return false
     */
    private function handleInbound(array $data)
    {
        if (!$this->validateData(['id', 'type', 'phone', 'text', 'sentAt', 'mediaUrl', 'mediaName'],  $data)) {
            return false;
        }

        $message = DB::getInstance()->getRow('SELECT id FROM '._DB_PREFIX_.'connectix_messages WHERE provider_id = "'.pSQL($data['id']).'"');
        if (!$message) {
            return true;
        }

        $values = [
            'message_id' => (int)$message['id'],
            'type' => pSQL($data['type']),
            'sent_at' => pSQL($data['sentAt'] ?? null),
        ];
        if (!empty($data['text'])) {
            $values['text'] = pSQL($data['text']);
        }
        if (!empty($data['mediaUrl']) && !empty($data['mediaName'])) {
            $values['media_url'] = pSQL($data['mediaUrl']);
            $values['media_name'] = pSQL($data['mediaName']);
        }

        try {
            Db::getInstance()->insert('connectix_message_inbounds', $values);
        } catch (PrestaShopDatabaseException $e) {
            return false;
        }

        return true;
    }

    /**
     * @param array $data
     * @return bool
     */
    private function handleTemplate(array $data)
    {
        if (!$this->validateData(['id', 'media', 'layout', 'name', 'channel', 'text', 'button_label', 'button_url', 'status'],  $data)) {
            return false;
        }

        $template = DB::getInstance()->getRow(
            'SELECT id FROM '._DB_PREFIX_.'connectix_templates WHERE provider_id = "'.pSQL($data['id']).'"'
        );

        $fields = ['media', 'layout', 'name', 'channel', 'text', 'button_label', 'button_url', 'status'];
        $fieldData = [];
        foreach ($fields as $field) {
            $fieldData[$field] = pSQL($data[$field]);
        }

        if ($template) {
            Db::getInstance()->update(
                'connectix_templates',
                $fieldData,
                'provider_id = "'.pSQL($data['id']).'"'
            );
        } else {
            $fieldData['provider_id'] = $data['id'];
            try {
                Db::getInstance()->insert('connectix_templates', $fieldData);
            } catch (PrestaShopDatabaseException $e) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param array $data
     * @return bool
     */
    private function handleReadyForDelivery(array $data)
    {
        $expected = ['trackingNumber', 'deliveryType'];
        if (isset($data['deliveryType']) && $data['deliveryType'] === 'door') {
            $expected[] = 'courier';
        } elseif (isset($data['deliveryType']) && $data['deliveryType'] === 'office') {
            $expected[] = 'office';
        }

        if (!$this->validateData($expected,  $data)) {
            return false;
        }

        try {
            $trackingOrders = DB::getInstance()->executeS(
                'SELECT order_id FROM '._DB_PREFIX_.'connectix_tracking WHERE tracking_number = "'.pSQL($data['trackingNumber']).'"'
            );
        } catch (PrestaShopDatabaseException $e) {
            return false;
        }

        $tmp = [];
        foreach ($trackingOrders as $trackingOrder) {
            try {
                $order = new Order((int)$trackingOrder['order_id']);
            } catch (PrestaShopDatabaseException $e) {
                continue;
            } catch (PrestaShopException $e) {
                continue;
            }
            if ($order) {
                $tmp[] = [
                    'order' => $order,
                    'deliveryData' => $data,
                ];
            }
        }

        if (!empty($tmp)) {
            try {
                Hook::exec(Connectix::HOOK_NAME_READY_FOR_DELIVERY, ['orders' => $tmp]);
            } catch (PrestaShopException $e) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param array $data
     * @return bool
     */
    private function handleDelivered(array $data)
    {
        if (empty($data['trackingNumber'])) {
            return false;
        }

        $deliveredStatus = (int)Configuration::get('CONNECTIX_DELIVERED_STATUS');
        if (!$deliveredStatus) {
            return true;
        }

        try {
            $trackingOrders = DB::getInstance()->executeS(
                'SELECT order_id FROM '._DB_PREFIX_.'connectix_tracking WHERE tracking_number = "'.pSQL($data['trackingNumber']).'"'
            );
        } catch (PrestaShopDatabaseException $e) {
            return false;
        }

        foreach ($trackingOrders as $trackingOrder) {
            try {
                $order = new Order((int)$trackingOrder['order_id']);
            } catch (PrestaShopDatabaseException $e) {
                continue;
            } catch (PrestaShopException $e) {
                continue;
            }

            if ($order && (int)$order->current_state !== $deliveredStatus) {
                $order->setCurrentState($deliveredStatus);
            }
        }

        return true;
    }

    /**
     * @param array $data
     * @return bool
     */
    private function handleNotPickedUp(array $data)
    {
        if (!$this->validateData(['trackingNumber', 'workingDaysPassed', 'office'],  $data)) {
            return false;
        }

        $rules = array_filter(Connectix::fetchRules(Connectix::TYPE_NOT_PICKED_UP), function ($rule) use ($data) {
            return (int)$data['workingDaysPassed'] === (int)$rule['days_not_picked_up'];
        });

        if (empty($data['trackingNumber'])) {
            return false;
        }

        if (empty($rules)) {
            return true;
        }

        try {
            $trackingOrders = DB::getInstance()->executeS(
                'SELECT order_id FROM '._DB_PREFIX_.'connectix_tracking WHERE tracking_number = "'.pSQL($data['trackingNumber']).'"'
            );
        } catch (PrestaShopDatabaseException $e) {
            return false;
        }

        $tmp = [];
        foreach ($trackingOrders as $trackingOrder) {
            try {
                $order = new Order((int)$trackingOrder['order_id']);
            } catch (PrestaShopDatabaseException $e) {
                continue;
            } catch (PrestaShopException $e) {
                continue;
            }

            $tmp[] = [
                'order' => $order,
                'deliveryData' => $data,
            ];
        }

        if (!empty($tmp)) {
            try {
                Hook::exec(Connectix::HOOK_NAME_NOT_PICKED_UP, [
                    'orders' => $tmp,
                    'rules' => $rules,
                ]);
            } catch (PrestaShopException $e) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param string $action
     * @param string $token
     */
    private function handleOrderConfirmation($action, $token)
    {
        $linksValidity = (int)Configuration::get('CONNECTIX_CONFIRMATION_URLS_VALIDITY');
        $expiredUrl = Configuration::get('CONNECTIX_CONFIRMATION_URLS_EXPIRED_URL');

        $actionUp = strtoupper($action);
        $enabled = (int)Configuration::get('CONNECTIX_'.$actionUp.'_ORDER');
        $orderStatus = (int)Configuration::get('CONNECTIX_'.$actionUp.'ED_ORDER_STATUS');
        $redirectUrl = Configuration::get('CONNECTIX_'.$actionUp.'ED_ORDER_URL');

        if (empty($redirectUrl)) {
            $redirectUrl = Context::getContext()->shop->getBaseURL(true);
        }

        if (!$enabled || !$orderStatus) {
            header("Location: $redirectUrl");
            exit;
        }

        $data = DB::getInstance()->getRow(
            'SELECT order_id, created_at FROM '._DB_PREFIX_.'connectix_order_token WHERE token = "'.pSQL($token).'"'
        );

        if ($data) {
            $diff = (time() - strtotime($data['created_at'])) / 3600;
            if (0 < $linksValidity && $linksValidity < $diff) {
                if (!$expiredUrl) {
                    echo json_encode('The link expired');
                }

                header("Location: $expiredUrl");
                exit;
            }

            try {
                $order = new Order((int)$data['order_id']);
            } catch (PrestaShopDatabaseException $e) {
                $order = null;
            } catch (PrestaShopException $e) {
                $order = null;
            }

            if ($order && (int)$order->current_state !== $orderStatus) {
                $order->setCurrentState($orderStatus);
            }

            DB::getInstance()->update('connectix_order_token', [
                'used' => 1,
                'status' => $action.'ed',
                'updated_at' => date('Y-m-d H:i:s', time()),
            ], 'token = "'.pSQL($token).'"');
        }

        header("Location: $redirectUrl");
        exit;
    }
}
