<?php
/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author PrestaShop SA <contact@prestashop.com>
 *  @copyright  2007-2020 PrestaShop SA
 *  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

declare(strict_types=1);

namespace PrestaShop\Module\Connectix\Install;

use Connectix;
use Db;
use Module;
use Hook;
use Configuration;
use PrestaShopDatabaseException;
use PrestaShopException;

/**
 * Class Installer
 * @package PrestaShop\Module\Connectix\Install
 */
class Installer
{
    /**
     * @var string[]
     */
    private $hooks = [
        'actionOrderStatusPostUpdate',
        'displayAdminOrderRight',
        'actionValidateOrder',
        'actionObjectOrderCarrierUpdateAfter',
    ];

    private $customHooks = [
        Connectix::HOOK_NAME_READY_FOR_DELIVERY,
        Connectix::HOOK_NAME_NOT_PICKED_UP,
    ];

    /**
     * @param Module $module
     * @return bool
     */
    public function install(Module $module): bool
    {
        if (!$this->installConfigDefaults()) {
            return false;
        }

        if (!$this->registerHooks($module)) {
            return false;
        }

        if (!$this->createHooks($module)) {
            return false;
        }

        if (!$this->installDatabase()) {
            return false;
        }

        return true;
    }

    /**
     * @param Module $module
     * @return bool
     */
    public function uninstall(Module $module): bool
    {
        if (!$this->uninstallConfigDefaults()) {
            return false;
        }

        if (!$this->unregisterHooks($module)) {
            return false;
        }

        if (!$this->deleteHooks($module)) {
            return false;
        }

        if (!$this->uninstallDatabase()) {
            return false;
        }

        return true;
    }

    /**
     * @return bool
     */
    public function installConfigDefaults()
    {
        // General settings
        Configuration::updateValue('CONNECTIX_ENABLED', 0);
        Configuration::updateValue('CONNECTIX_TOKEN', '');
        Configuration::updateValue('CONNECTIX_SYSTEM', 'sandbox');
        Configuration::updateValue('CONNECTIX_SECRET', md5(uniqid((string)mt_rand(), true)));
        Configuration::updateValue('CONNECTIX_ADD_CONTACTS', 1);
        Configuration::updateValue('CONNECTIX_TRACK_DELIVERY', 1);
        Configuration::updateValue('CONNECTIX_DELIVERED_STATUS', '');
        Configuration::updateValue('CONNECTIX_INTEGRATION_TOKEN', '');
        Configuration::updateValue('CONNECTIX_SANDBOX_URL', '');
        Configuration::updateValue('CONNECTIX_CONFIRM_ORDER', 0);
        Configuration::updateValue('CONNECTIX_CONFIRMED_ORDER_STATUS', '');
        Configuration::updateValue('CONNECTIX_CONFIRMED_ORDER_URL', '');
        Configuration::updateValue('CONNECTIX_CANCEL_ORDER', 0);
        Configuration::updateValue('CONNECTIX_CANCELED_ORDER_STATUS', '');
        Configuration::updateValue('CONNECTIX_CANCELED_ORDER_URL', '');
        Configuration::updateValue('CONNECTIX_CONFIRMATION_URLS_VALIDITY', '');
        Configuration::updateValue('CONNECTIX_CONFIRMATION_URLS_EXPIRED_URL', '');
        Configuration::updateValue('CONNECTIX_HOOK_CALLBACK', '');
        Configuration::updateValue('CONNECTIX_HOOK_INBOUND', '');
        Configuration::updateValue('CONNECTIX_HOOK_TEMPLATE', '');
        Configuration::updateValue('CONNECTIX_HOOK_TRACKING', '');

        return true;
    }

    /**
     * @return bool
     */
    public function uninstallConfigDefaults()
    {
        // General settings
        Configuration::deleteByName('CONNECTIX_ENABLED');
        Configuration::deleteByName('CONNECTIX_TOKEN');
        Configuration::deleteByName('CONNECTIX_SYSTEM');
        Configuration::deleteByName('CONNECTIX_SECRET');
        Configuration::deleteByName('CONNECTIX_ADD_CONTACTS');
        Configuration::deleteByName('CONNECTIX_TRACK_DELIVERY');
        Configuration::deleteByName('CONNECTIX_DELIVERED_STATUS');
        Configuration::deleteByName('CONNECTIX_INTEGRATION_TOKEN');
        Configuration::deleteByName('CONNECTIX_SANDBOX_URL');
        Configuration::deleteByName('CONNECTIX_CONFIRM_ORDER');
        Configuration::deleteByName('CONNECTIX_CONFIRMED_ORDER_STATUS');
        Configuration::deleteByName('CONNECTIX_CONFIRMED_ORDER_URL');
        Configuration::deleteByName('CONNECTIX_CANCEL_ORDER');
        Configuration::deleteByName('CONNECTIX_CANCELED_ORDER_STATUS');
        Configuration::deleteByName('CONNECTIX_CANCELED_ORDER_URL');
        Configuration::deleteByName('CONNECTIX_CONFIRMATION_URLS_VALIDITY');
        Configuration::deleteByName('CONNECTIX_CONFIRMATION_URLS_EXPIRED_URL');
        Configuration::deleteByName('CONNECTIX_HOOK_CALLBACK', '');
        Configuration::deleteByName('CONNECTIX_HOOK_INBOUND', '');
        Configuration::deleteByName('CONNECTIX_HOOK_TEMPLATE', '');
        Configuration::deleteByName('CONNECTIX_HOOK_TRACKING', '');

        return true;
    }

    /**
     * Install the database modifications required for this module.
     *
     * @return bool
     */
    private function installDatabase(): bool
    {
        $queries = [];

        include_once(dirname(__FILE__).'/sql/install.php');

        if (empty($queries)) {
            return true;
        }

        return $this->executeQueries($queries);
    }

    /**
     * Uninstall database modifications.
     *
     * @return bool
     */
    private function uninstallDatabase(): bool
    {
        $queries = [];
        include_once(dirname(__FILE__).'/sql/uninstall.php');

        if (empty($queries)) {
            return true;
        }

        return $this->executeQueries($queries);
    }

    /**
     * Register hooks for the module.
     *
     * @param Module $module
     *
     * @return bool
     */
    private function registerHooks(Module $module): bool
    {
        $pass = true;
        foreach ($this->hooks as $hook) {
            if (!(bool)$module->registerHook($hook)) {
                $pass = false;
                break;
            }
        }

        return $pass;
    }

    /**
     * Register hooks for the module.
     *
     * @param Module $module
     *
     * @return bool
     */
    private function unregisterHooks(Module $module): bool
    {
        $pass = true;
        foreach ($this->hooks as $hook) {
            if (!(bool)$module->unregisterHook($hook)) {
                $pass = false;
                break;
            }
        }

        return $pass;
    }

    /**
     * @param Module $module
     * @return bool
     */
    public function createHooks(Module $module)
    {
        $success = true;
        foreach ($this->customHooks as $customHook) {
            if (!Hook::getIdByName($customHook)) {
                $hook = new Hook();
                $hook->name = $customHook;

                try {
                    if (!$hook->add()) {
                        return false;
                    }
                } catch (PrestaShopDatabaseException $e) {
                    return false;
                } catch (PrestaShopException $e) {
                    return false;
                }
            }

            if (!$success) {
                break;
            }

            $success = $module->registerHook($customHook);
        }


        return $success;
    }

    /**
     * @param Module $module
     * @return bool
     */
    public function deleteHooks(Module $module)
    {
        $success = true;
        foreach ($this->customHooks as $customHook) {
            $hookId = (int)Hook::getIdByName($customHook);
            if (!$hookId) {
                continue;
            }

            try {
                $hook = new Hook($hookId);
            } catch (PrestaShopDatabaseException $e) {
                return false;
            } catch (PrestaShopException $e) {
                return false;
            }

            try {
                if (!$hook->delete()) {
                    return false;
                }
            } catch (PrestaShopException $e) {
                return false;
            }

            if (!$success) {
                break;
            }

            $success = $module->unregisterHook($customHook);
        }

        return $success;
    }

    /**
     * A helper that executes multiple database queries.
     *
     * @param array $queries
     *
     * @return bool
     */
    private function executeQueries(array $queries): bool
    {
        foreach ($queries as $query) {
            try {
                Db::getInstance()->executeS($query);
            } catch (PrestaShopDatabaseException $e) {
                return false;
            }
        }

        return true;
    }
}
