<?php

namespace Connectix\Components;

use Connectix\Exception\RequiredValueException;
use Connectix\Exception\ValueException;

final class Message extends BaseComponent
{
	/**
	 * @var int|null
	 */
	protected $ttl;

	/**
	 * @var string|null
	 */
	protected $callbackUrl;

	/**
	 * @var string|null
	 */
	protected $inboundUrl;

	/**
	 * @var string|null
	 */
	protected $template;

	/**
	 * @var array
	 */
	protected $parameters = [];

	/**
	 * @var Contact|null
	 */
	protected $contact;

	/**
	 * @var string|null
	 */
	protected $phone;

	/**
	 * @var int|null
	 */
	protected $position;

	/**
	 * @var bool
	 */
	protected $compound = false;

	/**
	 * @inheritDoc
	 */
	public function getRequiredFields()
	{
		$fields = ['template'];
		if ($this->isCompound()) {
			$fields[] = 'position';
		} else {
			$fields[] = 'phone';
		}

		return $fields;
	}

	/**
	 * @throws RequiredValueException
	 * @throws ValueException
	 */
	protected function validate()
	{
		parent::validate();

		if (!is_string($this->getTemplate()) || !preg_match('/^[a-f\d]{8}(-[a-f\d]{4}){4}[a-f\d]{8}$/i', $this->getTemplate())) {
			throw new ValueException("Value \"".$this->getTemplate()."\" is not a valid template ID.");
		}

		if ($this->getCallbackUrl() && false === filter_var($this->getCallbackUrl(), FILTER_VALIDATE_URL)) {
			throw new ValueException("Value \"".$this->getCallbackUrl()."\" is not a valid URL.");
		}

		if ($this->getInboundUrl() && false === filter_var($this->getInboundUrl(), FILTER_VALIDATE_URL)) {
			throw new ValueException("Value \"".$this->getInboundUrl()."\" is not a valid URL.");
		}
	}

	/**
	 * @return string[]
	 * @throws RequiredValueException
	 * @throws ValueException
	 */
	public function toArray()
	{
		$this->validate();

		$params = [
			'template' => $this->getTemplate(),
		];

		if (null !== $this->getTtl()) {
			$params['ttl'] = $this->getTtl();
		}

		if (null !== $this->getCallbackUrl()) {
			$params['callbackUrl'] = $this->getCallbackUrl();
		}

		if (null !== $this->getInboundUrl()) {
			$params['inboundUrl'] = $this->getInboundUrl();
		}

		if (!empty($this->getParameters())) {
			$params['parameters'] = $this->getParameters();
		}

		if ($this->isCompound()) {
			$params['position'] = $this->getPosition();
		} else {
			$params['phone'] = $this->getPhone();

			if (null !== $this->getContact()) {
				$params['contact'] = $this->getContact()->toArray();
			}
		}

		return $params;
	}

	/**
	 * @inheritDoc
	 */
	public function toJson()
	{
		if ($data = json_encode($this->toArray())) {
			return $data;
		}

		return '';
	}

	/**
	 * @return int|null
	 */
	public function getTtl()
	{
		return $this->ttl;
	}

	/**
	 * @param int $ttl
	 * @return Message
	 */
	public function setTtl($ttl)
	{
		$this->ttl = $ttl;

		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getCallbackUrl()
	{
		return $this->callbackUrl;
	}

	/**
	 * @param string|null $callbackUrl
	 * @return Message
	 */
	public function setCallbackUrl($callbackUrl)
	{
		$this->callbackUrl = $callbackUrl;

		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getInboundUrl()
	{
		return $this->inboundUrl;
	}

	/**
	 * @param string|null $inboundUrl
	 * @return Message
	 */
	public function setInboundUrl($inboundUrl)
	{
		$this->inboundUrl = $inboundUrl;

		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getTemplate()
	{
		return $this->template;
	}

	/**
	 * @param string $template
	 * @return Message
	 */
	public function setTemplate($template)
	{
		$this->template = $template;

		return $this;
	}

	/**
	 * @return array
	 */
	public function getParameters()
	{
		return $this->parameters;
	}

	/**
	 * @param array $parameters
	 * @return Message
	 */
	public function setParameters($parameters)
	{
		// Make sure that all parameters are strings
		$this->parameters = array_map(function ($param) {
			return (string)$param;
		}, $parameters);

		return $this;
	}

	/**
	 * @return Contact|null
	 */
	public function getContact()
	{
		return $this->contact;
	}

	/**
	 * @param Contact|null $contact
	 * @return Message
	 */
	public function setContact($contact)
	{
		$this->contact = $contact;

		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getPhone()
	{
		return $this->phone;
	}

	/**
	 * @param string|null $phone
	 * @return Message
	 */
	public function setPhone($phone)
	{
		$this->phone = $phone;

		return $this;
	}

	/**
	 * @return int|null
	 */
	public function getPosition()
	{
		return $this->position;
	}

	/**
	 * @param int|null $position
	 * @return Message
	 */
	public function setPosition($position)
	{
		$this->position = $position;

		return $this;
	}

	/**
	 * @return bool
	 */
	public function isCompound(): bool
	{
		return $this->compound;
	}

	/**
	 * @param bool $compound
	 * @return Message
	 */
	public function setCompound($compound)
	{
		$this->compound = $compound;

		if ($compound) {
			$this->setPhone(null);
			$this->setContact(null);
		}

		return $this;
	}
}
